/*
  +----------------------------------------------------------------------+
  | Copyright (c) 2009-2013 The PHP Group                                |
  +----------------------------------------------------------------------+
  | This source file is subject to version 3.01 of the PHP license,      |
  | that is bundled with this package in the file LICENSE, and is        |
  | available through the world-wide-web at the following url:           |
  | http://www.php.net/license/3_01.txt.                                 |
  | If you did not receive a copy of the PHP license and are unable to   |
  | obtain it through the world-wide-web, please send a note to          |
  | license@php.net so we can mail you a copy immediately.               |
  +----------------------------------------------------------------------+
  | Authors: Mikko Koppanen <mkoppanen@php.net>                          |
  +----------------------------------------------------------------------+
*/

#ifndef _PHP_MEMCACHED_SERVER_H_
# define _PHP_MEMCACHED_SERVER_H_

#ifdef HAVE_MEMCACHED_PROTOCOL

#include <libmemcachedprotocol-0.0/handler.h>

/*
	Opaque structure
*/
typedef struct _php_memc_proto_handler_t php_memc_proto_handler_t;

/*
	Functions
*/
php_memc_proto_handler_t *php_memc_proto_handler_new ();

void php_memc_proto_handler_destroy (php_memc_proto_handler_t **ptr);

zend_bool php_memc_proto_handler_run (php_memc_proto_handler_t *h, zend_string *address);

#endif

#endif
